/******************************************************************************
*
* project name:    TICHESS
* project version: 2.00
* file name:       gui.c
* initial date:    24/02/2000
* author:          thomas.nussbaumer@peem.com
* description:     contains most of the GUI stuff
*
* -----------------------------------------------------------------------------
* Revision History
* -----------------------------------------------------------------------------
*
* v1.22: 10/04/2000
*        promotion text shortened to fit on TI89 output line
*        display of stored moves may be canceled by [ESC]
*
* v1.21: 07/04/2000
*        coordinates will displayed now correctly after board rotation
*
* v1.20: 06/04/2000
*        Overwriting of the "N:" fixed
*
* v1.19: 05/04/2000
*        no alpha key necessary anymore for promotion (TI89)
*        some splitted messages now correctly splitted (TI89)
*        stored moves output now a little bit more pretty (TI89)
*
* v1.18: 03/04/2000
*        objective of chess puzzle get printed after "puzzle xxx loaded" output
*        move list output fixed for TI89
*
* v1.17: 29/03/2000
*        wrap arround of cursor added
*        inner contents of figures will now be masked
*
* v1.16: 27/03/2000
*        no more ALPHA key necessary on TI89
*        load & save combined to load/save dialog
*        'p' to select 2 player on TI92 removed
*
* v1.15: 24/03/2000
*        now using popup for requests
*
* v1.14: 22/03/2000
*        disabled now selection of a figure if already mate
*        putc(): handling of '\r' fixed (quick return)
*        treatment of new setting entry "REQUESTS" (disabling of Y/N requests)
*        hides cursor during computer move
*
* v1.13: 21/03/2000
*        many things restructured due to the integration of the graphical
*        menu system
*
* v1.12: 19/03/2000
*        save function now stores only board and system flags
*        load/save function extracted to file loadsave.c
*        special key [K] added (show hashkey)
*
* v1.11: 17/03/2000
*        [DIAMOND] [OFF] for turn calculator off added
*        special key [M] added (toggle hash mode)
*        special key [Z] added (toggle hash statistics)
*
* v1.10: 11/03/2000
*        won't anounce "CHECK !!" anymore if already checkmate
*        can use [F1] instead of [ENTER] on TI92
*
* v1.09: 10/03/2000
*        monochrome version support added
*        hashtable support added
*        marks now the from-field, too
*
* v1.08: 09/03/2000
*        engine will say check (not in 2player mode)
*
* v1.07: 08/03/2000
*        move display corrected
*        paints now rectangle arround last move
*        help page of TI-92 fixed
*
* v1.06: 07/03/2000
*        unnecessary includes removed
*        using now include files for the piece art (thanx to Marianna Csillag
*              for the piece art and Kristof Csillag for the ideas and tools)
*        using now [STO] and [BACK] keys
*        [ESC] can be used instead of 'N'
*        calculator can now be turned off during user move
*        some outputs modified
*        [CLEAR] or [HOME] can be used instead of ENTER on the TI89
*
* v1.05: 02/03/2000
*        ported to TI89
*
* v1.04: 01/03/2000
*        graphical artifacts removed (line height increased)
*        two real player mode added (key 'p')
*
* v1.03: 29/02/2000
*        new function HandleComputerMove() for more clearance
*
* v1.02: 28/02/2000
*        UpdateMoveDisplay() fixed for rochade display (won't left garbage)
*
* v1.01: 27/02/2000
*        UpdateMoveDisplay() fixed for rochade display (won't left garbage)
*
* v1.00: 24/02/2000
*        extracted from tichess.c
*
******************************************************************************/

#if TI92_VERSION
//-----------------------------------------------------------------------------
// GRAPHICAL CONSTANTS FOR TI92plus
//-----------------------------------------------------------------------------
#define DISPLAY_TURN_X   126     // x position of turn display
#define DISPLAY_TURN_Y     1     // y position of turn display
#define DISPLAY_TURN_W   "white"
#define DISPLAY_TURN_B   "black"
#define DISPLAY_MOVE_X   168     // x position of move display
#define DISPLAY_MOVE_Y     1     // y position of move display
#define DISPLAY_NODE_X   215     // x position of node display
#define DISPLAY_NODE_Y     1     // y position of node display
#define COORD_POS_X      112
#define COORD_POS_Y      112
#define OFFSET_X           0     // x offset of board display
#define OFFSET_Y           0     // y offset of board display
#define SIZE_X            14     // x field size of board display
#define SIZE_Y            14     // y field size of board display
#define TXT_START_X      120     // x text start position
#define TXT_START_Y       12     // y text start position
#define TXT_HEIGHT         8     // height of a text line
#define LINES_PER_PAGE    11     // minimum lines per page
#define TXT_END_X        239     // x text end position
#define TXT_END_Y        119     // y text end position
#define COORD_OFFSET_X   (OFFSET_X+SIZE_X/3)
#define COORD_OFFSET_Y   (OFFSET_Y+SIZE_Y/3)
#define KEY_ESCAPE       264
#define KEY_LEFT         337
#define KEY_RIGHT        340
#define KEY_UP           338
#define KEY_DOWN         344
#define KEY_ENTER         13
#define KEY_OFF1        4363
#define KEY_OFF2        8459
#define KEY_HELP         'h'
#define KEY_FORWARD      'f'
#define KEY_BACK         257    // [backspace]
#define KEY_LOADSAVE     258    // [STO]
#define KEY_LEVELUP      '+'
#define KEY_LEVELDOWN    '-'
#define KEY_F1           268
#define KEY_AUTOMATIC    'a'

#else
//-----------------------------------------------------------------------------
// GRAPHICAL CONSTANTS FOR TI89
//-----------------------------------------------------------------------------
#define DISPLAY_TURN_X    20     // x position of turn display
#define DISPLAY_TURN_Y    87     // y position of turn display
#define DISPLAY_TURN_W   "Turn: White"
#define DISPLAY_TURN_B   "Turn: Black"
#define DISPLAY_MOVE_X    93     // x position of move display
#define DISPLAY_MOVE_Y     1     // y position of move display
#define DISPLAY_NODE_X   129     // x position of node display
#define DISPLAY_NODE_Y     1     // y position of node display
#define COORD_POS_X       81
#define COORD_POS_Y       81
#define OFFSET_X           0     // x offset of board display
#define OFFSET_Y           0     // y offset of board display
#define SIZE_X            10     // x field size of board display
#define SIZE_Y            10     // y field size of board display
#define TXT_START_X       87     // x text start position
#define TXT_START_Y       10     // y text start position
#define TXT_HEIGHT         6     // height of a text line
#define LINES_PER_PAGE    12     // minimum lines per page

#define TXT_END_X        158     // x text end position
#define TXT_END_Y         92     // y text end position

#define COORD_OFFSET_X   (OFFSET_X+SIZE_X/3)
#define COORD_OFFSET_Y   (OFFSET_Y+SIZE_Y/3)
#define KEY_ESCAPE       264
#define KEY_LEFT         338
#define KEY_RIGHT        344
#define KEY_UP           337
#define KEY_DOWN         340
#define KEY_ENTER         13
#define KEY_OFF1        4363
#define KEY_OFF2       16651
#define KEY_CLEAR        263
#define KEY_HOME         277
#define KEY_HELP         268    // F1
#define KEY_FORWARD      278    // CATALOG
#define KEY_BACK         257    // [backspace]
#define KEY_LOADSAVE     258    // [STO]
#define KEY_LEVELUP      '+'
#define KEY_LEVELDOWN    '-'
#define KEY_AUTOMATIC    '0'

#endif

//-----------------------------------------------------------------------------
// GLOBALS
//-----------------------------------------------------------------------------

int      __x            =  TXT_START_X; // actual x position of text output
int      __y            =  TXT_START_Y; // actual y position of text output
SCR_RECT squarerect     = {};           // temporary rectangle
int      inverted       = FALSE;
int      select_x       =  0;           // selected square (x coordinate)
int      select_y       =  0;           // selected square (y coordinate)
int      marked_rect1   = -1;
int      marked_rect2   = -1;
int      idx_from       = -1;           // index into board of from-square
int      idx_to         = -1;           // index into board of to-square
int      select_promote = 0;            // used to store selected promotion fig
int      automatic_mode = FALSE;        // automatic mode setting
int      twoplayer_mode = FALSE;        // two player mode setting
char     jmp_buf[56]    = {};           // for longjump usage (exit or restart)
char     do_exit        = 0;            // for exit() handling
char     drawn[BOARD_SIZE] = {};        // used to update board display
int      choice         = MENU_NONE;
int      nr_planes      = 2;            // standard: grayscale 
int      firsttime      = 1;            // used within coordinates output

//-----------------------------------------------------------------------------
// declarations of the necessary "external" routines and variables
//-----------------------------------------------------------------------------

extern char*         board;      // the board
extern int           act_color;  // which turn (white or black)
extern int           std_depth;  // standard depth
extern unsigned int  move_count; // number of moves so far
extern long          nr_nodes;
extern int           white_king_pos;
extern int           black_king_pos;
extern hash_t        act_hashkey;
extern int           already_mate;
extern store_t*      move_store;
extern int           moves_stored;
extern int           moves_forward;
extern defaults_t    defaults;
extern int           problem_active;
extern char          problem_text1[SIZE_TEXT+1];
extern char          problem_text2[SIZE_TEXT+1];
extern char          problem_text3[SIZE_TEXT+1];

int            AttacksField(int field,int side);
char*          Move2Str(move_t* pmove,int fromboard);
char*          Field2Notation(int fieldnumber,char* fieldnotation);
char           Figure2Char(char figure);
move_t*        ComputerMove(void);
int            UserMove(char* move);
void           InitEngine(int level);
void           TakeBackMove(int silent);
void           UndoTakeBackMove(int silent);
void           CalcHashKey(void);
inline void    InitHashCode(void);

#if DEVELOPER_VERSION

char*          Hash2Str(hash_t code);
inline void    ToggleHashMode(void);
inline void    ToggleHashStatistics(void);
int            Evaluate(int alpha,int beta,int side);
extern int*    material_bal;
extern int*    material_sum;
extern int     act_depth;

#endif

int            HandleLoad(int automatic,int load_problem);
int            HandleSave(int automatic);
void           OptionMenu();
int            SaveDefaults();
int            MainMenu(int showmenu);

#if TI92_VERSION
//-----------------------------------------------------------------------------
// ICONS FOR TI92
//-----------------------------------------------------------------------------
ICON bicons[12] = {
#include "black92.ico"
,
#include "m_b92.ico"
};
ICON wicons[12] = {
#include "white92.ico"
,
#include "m_w92.ico"
};
#else
//-----------------------------------------------------------------------------
// ICONS FOR TI89
//-----------------------------------------------------------------------------
ICON bicons[12] = {
#include "black89.ico"
,
#include "m_b89.ico"
};
ICON wicons[12] = {
#include "white89.ico"
,
#include "m_w89.ico"
};
#endif


/*===========================================================================*/
/* helper function for detecting a 'Y' press                                 */
/*===========================================================================*/
int isYesKey(int key) {
    if (key == 'Y'       ||
        key == 'y'       ||
#if TI92_VERSION
        key == KEY_F1    ||
#else
        key == KEY_CLEAR ||
        key == KEY_HOME  ||
#endif
        key == KEY_ENTER)
    {
        return TRUE;
    }
    else {
        return FALSE;
    }
}


/*===========================================================================*/
/* helper function for detecting a 'N' press                                 */
/*===========================================================================*/
int isNoKey(int key) {
    if (key == 'N' || key == 'n' || key == KEY_ESCAPE)
    {
        return TRUE;
    }
    else {
        return FALSE;
    }
}


/*===========================================================================*/
/* printf output to terminal window                                          */
/*===========================================================================*/
void HandleSetPlane(int i) {
    if (nr_planes == 2) SetPlane(i);
}


/*===========================================================================*/
/* printf output to terminal window                                          */
/*===========================================================================*/
#define printf(format,args...) \
{char printbuffer[200]; sprintf(printbuffer,format,##args);puts(printbuffer);}


/*===========================================================================*/
/* outputs one character to the terminal window                              */
/*===========================================================================*/
void putc(char c) {
    int i,j;
    char buffer[200];

    //--------------------------------------------------
    // clears line and positions cursor at the beginning
    //--------------------------------------------------
    if (c=='\r') {
        __x = TXT_START_X;
        i = (TXT_END_X-TXT_START_X)/FontCharWidth(' ');
        for (j=0;j<i;j++) buffer[j] = ' ';
        buffer[j] = 0;

        for (i=0;i<nr_planes;i++) {
            HandleSetPlane(i);
            DrawStr(__x,__y,buffer,A_REPLACE);
        }
        return;
    }

    //------------------------------------------
    // wrap arround if necessary
    //------------------------------------------
    if (c=='\n'||__x+FontCharWidth(c)>TXT_END_X)
        __x=TXT_START_X,__y+=TXT_HEIGHT;

    //----------------------------------------------
    // don't scroll if it's not really necessary
    //----------------------------------------------
    if (c == '\n') return;

    if (__y+TXT_HEIGHT+3>=TXT_END_Y) {
        for (i=0;i<nr_planes;i++) {
            HandleSetPlane(i);
            ScrRectScroll(&(SCR_RECT){TXT_START_X,TXT_START_Y,TXT_END_X,TXT_END_Y},
                          ScrRect,TXT_HEIGHT,A_REVERSE);
        }
        __y-=TXT_HEIGHT;
    }

    for (i=0;i<nr_planes;i++) {
        HandleSetPlane(i);
        DrawChar(__x,__y,c,A_REPLACE);
    }

    __x+=FontCharWidth(c);
}


/*===========================================================================*/
/* outputs a string to terminal window                                       */
/*===========================================================================*/
void puts(const char *s) {
    while (*s) putc(*s++);
}


/*===========================================================================*/
/* simple exit() implementation which does a long jump back into MainGUI()   */
/*===========================================================================*/
void LeavePrg(char* msg) {
    if (msg) {
        puts(msg);
        puts("-------- PRESS KEY -------\n");
        ngetchx();
    }
    do_exit = 1;
    longjmp(jmp_buf,1);
}


/*===========================================================================*/
/* update the "TURN OF" display                                              */
/*===========================================================================*/
void UpdateTurnDisplay(int color) {
    int  i;

    for (i=0;i<nr_planes;i++) {
        HandleSetPlane(i);
        if (color == WHITE) DrawStr(DISPLAY_TURN_X,DISPLAY_TURN_Y,DISPLAY_TURN_W,A_REPLACE);
        else                DrawStr(DISPLAY_TURN_X,DISPLAY_TURN_Y,DISPLAY_TURN_B,A_REPLACE);
    }
}


/*===========================================================================*/
/* update the "BEST MOVE SO FAR" display                                     */
/*===========================================================================*/
void UpdateMoveDisplay(move_t* pmove) {
    char s[20];
    int  i;
    sprintf(s,"%s     ",Move2Str(pmove,TRUE));

    for (i=0;i<nr_planes;i++) {
        HandleSetPlane(i);
        DrawStr(DISPLAY_MOVE_X,DISPLAY_MOVE_Y,s,A_REPLACE);
    }
}


/*===========================================================================*/
/* update the "NODES EXAMINED" display                                       */
/*===========================================================================*/
void UpdateNodeDisplay(long nodes) {
    char s[10];
    int  i;
#if TI92_VERSION
    sprintf(s,"%06ld",nodes);

    for (i=0;i<nr_planes;i++) {
        HandleSetPlane(i);
        DrawStr(DISPLAY_NODE_X,DISPLAY_NODE_Y,s,A_REPLACE);
    }
#else
    sprintf(s,"N:%06ld",nodes);

    for (i=0;i<nr_planes;i++) {
        HandleSetPlane(i);
        DrawStr(DISPLAY_NODE_X,DISPLAY_NODE_Y,s,A_REPLACE);
    }
#endif    
}


/*===========================================================================*/
/* draw coordinates                                                          */
/*===========================================================================*/
void DrawCoordinates() {
    int i,j;

    for (i=0;i<nr_planes;i++) {
        HandleSetPlane(i);

        for (j=0;j<8;j++) {
            if (!inverted) {
                if (!firsttime) {
                    DrawChar(COORD_OFFSET_X+j*SIZE_X,COORD_POS_Y,'h'-j,A_XOR);
                    DrawChar(COORD_POS_X,COORD_OFFSET_Y+j*SIZE_Y,'8'-7+j,A_XOR);
                }
                DrawChar(COORD_OFFSET_X+j*SIZE_X,COORD_POS_Y,'a'+j,A_REPLACE);
                DrawChar(COORD_POS_X,COORD_OFFSET_Y+j*SIZE_Y,'1'+7-j,A_REPLACE);
            }
            else {
                if (!firsttime) {
                    DrawChar(COORD_OFFSET_X+j*SIZE_X,COORD_POS_Y,'a'+j,A_XOR);
                    DrawChar(COORD_POS_X,COORD_OFFSET_Y+j*SIZE_Y,'1'+7-j,A_XOR);
                }
                DrawChar(COORD_OFFSET_X+j*SIZE_X,COORD_POS_Y,'h'-j,A_REPLACE);
                DrawChar(COORD_POS_X,COORD_OFFSET_Y+j*SIZE_Y,'8'-7+j,A_REPLACE);
            }
        }
    }
    firsttime = 0;
}


/*===========================================================================*/
/* draws marking rectangle arround given square (last computer move)         */
/*===========================================================================*/
void DrawMarkingRect(int idx) {
    WIN_RECT sq;
    int      i;
    int      x;
    int      y;

    if (inverted) {
        y = idx / 10 - 2;
        x = 7 - (idx % 10 - 1);
    }
    else {
        y = 7 - (idx / 10 - 2);
        x = idx % 10 - 1;
    }

    sq.x0 = OFFSET_X+SIZE_X*x;
    sq.x1 = OFFSET_X+SIZE_X*(x+1)-1;
    sq.y0 = OFFSET_Y+SIZE_Y*y;
    sq.y1 = OFFSET_Y+SIZE_Y*(y+1)-1;

    for (i=0;i<nr_planes;i++) {
        HandleSetPlane(i);
        DrawClipRect(&sq,ScrRect,A_REPLACE);
    }
}


/*===========================================================================*/
/* set rectangle of field x,y                                                */
/*===========================================================================*/
void SetRect2Square(int x,int y) {
    squarerect.xy.x0 = OFFSET_X+SIZE_X*x;
    squarerect.xy.x1 = OFFSET_X+SIZE_X*(x+1)-1;
    squarerect.xy.y0 = OFFSET_Y+SIZE_Y*y;
    squarerect.xy.y1 = OFFSET_Y+SIZE_Y*(y+1)-1;
}


/*===========================================================================*/
/* maps cursor position to field value                                       */
/*===========================================================================*/
int Cursor2Idx(int x,int y) {
    if (inverted) return 21 + y*10+(7-x);
    else          return 21 + (7-y)*10+x;
}


/*===========================================================================*/
/* draws single cell                                                         */
/*===========================================================================*/
void DrawCellGray(int x, int y,char s) {
    int i;
    ICON* picon;
    ICON* micon;

    if (s < 0) {
        picon = &bicons[abs(s+1)];
        micon = &bicons[abs(s+1)+6];
    }
    else {
        picon = &wicons[s-1];
        micon = &wicons[s-1+6];
    }

    SetRect2Square(x,y);

    for (i=0;i<2;i++) {
        SetPlane(i);
        if (i==0 && ((y*8+x)%2)!=y%2) {
           ScrRectFill(&squarerect,ScrRect,A_REPLACE);
        }
        else {
           ScrRectFill(&squarerect,ScrRect,A_REVERSE);
        }
        if (s != EMPTY) {
#if TI92_VERSION
            DrawIcon(x*SIZE_X+OFFSET_X+1,y*SIZE_Y+OFFSET_Y+2,micon,A_REVERSE);
            DrawIcon(x*SIZE_X+OFFSET_X+1,y*SIZE_Y+OFFSET_Y+2,picon,A_REPLACE);
#else
            DrawIcon(x*SIZE_X+OFFSET_X,y*SIZE_Y+OFFSET_Y,micon,A_REVERSE);
            DrawIcon(x*SIZE_X+OFFSET_X,y*SIZE_Y+OFFSET_Y,picon,A_REPLACE);
#endif
        }
    }
}

#if TI92_VERSION
ICON bckgrd = {0x8888,0,0,0,0x8888,0,0,0,0x8888,0,0,0,0x8888,0,0,0};
#else
ICON bckgrd = {0x1200,0,0,0x9240,0,0,0x9240,0,0,0x1200,0,0,0,0,0,0};
//ICON bckgrd = {0x9240,0,0,0x9240,0,0,0x9240,0,0,0x9240,0,0,0,0,0,0};
#endif

void DrawCellMono(int x, int y,char s) {
    ICON* picon;
    ICON* micon;

    if (s < 0) {
        picon = &bicons[abs(s+1)];
        micon = &bicons[abs(s+1)+6];
    }
    else {
        picon = &wicons[s-1];
        micon = &wicons[s-1+6];
    }

    SetRect2Square(x,y);

    if (((y*8+x)%2)!=y%2) {
       ScrRectFill(&squarerect,ScrRect,A_REVERSE);
       DrawIcon(x*SIZE_X+OFFSET_X,y*SIZE_Y+OFFSET_Y,&bckgrd,A_REPLACE);
    }
    else {
       ScrRectFill(&squarerect,ScrRect,A_REVERSE);
    }

    if (s != EMPTY) {
#if TI92_VERSION
        DrawIcon(x*SIZE_X+OFFSET_X+1,y*SIZE_Y+OFFSET_Y+2,micon,A_REVERSE);
        DrawIcon(x*SIZE_X+OFFSET_X+1,y*SIZE_Y+OFFSET_Y+2,picon,A_REPLACE);
#else
        DrawIcon(x*SIZE_X+OFFSET_X,y*SIZE_Y+OFFSET_Y,micon,A_REVERSE);
        DrawIcon(x*SIZE_X+OFFSET_X,y*SIZE_Y+OFFSET_Y,picon,A_REPLACE);
#endif
    }

}


/*===========================================================================*/
/* draws complete board                                                      */
/*===========================================================================*/
void DrawBoard(int hidecursor) {
    int i,j,idx;
    char val;

    // update side to draw state display
    UpdateTurnDisplay(act_color);


    // check all squares for redraw ...
    for (i=0;i<8;i++) {
        for (j=0;j<8;j++) {
            idx = Cursor2Idx(i,j);

            val = board[idx];
            if (drawn[idx] == val) {
                if (i!=select_x || j!=select_y) continue;
            }
            if (nr_planes == 2) DrawCellGray(i,j,val);
            else                DrawCellMono(i,j,val);

            drawn[idx] = val;

            if (idx == marked_rect1) {
               DrawMarkingRect(idx);
               drawn[idx]   = -OUTSIDE;
               marked_rect1 = -1;
            }
            else if (idx == marked_rect2) {
               DrawMarkingRect(idx);
               drawn[idx]   = -OUTSIDE;
               marked_rect2 = -1;
            }
        }
    }

    if (!hidecursor) {
        SetRect2Square(select_x,select_y);
        for (i=0;i<nr_planes;i++) {
            HandleSetPlane(i);
            ScrRectFill(&squarerect,ScrRect,A_XOR);
        }
    }
}


/*===========================================================================*/
/* initialize GUI (NOTE: GRAYMODE HAVE TO BE SET OUTSIDE !!!)                */
/*===========================================================================*/
void InitGUI() {
    int i,j;

    __x = TXT_START_X;
    __y = TXT_START_Y;

    select_x     =  0;
    select_y     =  7;
    idx_from     = -1;
    idx_to       = -1;
    marked_rect1 = -1;
    marked_rect2 = -1;

    FontSetSys(F_4x6);

    firsttime = 1;

    for (i=0;i<nr_planes;i++) {
        HandleSetPlane(i);
        ClrScr();
        DrawLine(TXT_START_X-1,7,TXT_END_X,7,A_REPLACE);
        DrawLine(TXT_START_X-2,0,TXT_START_X-2,TXT_END_Y+1,A_REPLACE);
        DrawLine(0,TXT_END_Y+1,TXT_END_X,TXT_END_Y+1,A_REPLACE);
#if TI92_VERSION
        DrawStr(TXT_START_X,1,"T:xxxxx        B:-------      N:000000",A_REPLACE);
        DrawStr(0,122,"TI-CHESS "TIC_VERSION_MAIN"              [press H for Help]              (c) the TI-CHESS TEAM",A_REPLACE);
#else
        DrawStr(TXT_START_X,1,"B:-------    N:000000",A_REPLACE);
        //---------------------------------------------------------------------
        // Due to a really strange problem the bottom line MUST look like this.
        // (A part of the bottom line directly after the TI-CHESS string gets
        //  otherwise erased !!)
        //---------------------------------------------------------------------
        DrawStr(0,95,"TI-CHESS            [press F1 for Help]            v"TIC_VERSION_MAIN,A_REPLACE);
#endif
    }

    for (i=0;i<BOARD_SIZE;i++) drawn[i] = -OUTSIDE;

    DrawCoordinates();
    InitHashCode();
}


/*===========================================================================*/
/* handles a request dialog (returns TRUE if 'y' is pressed)                 */
/*===========================================================================*/
int RequestDialog(char* request) {
     if (defaults.requests == OPTION_OFF) {
         return TRUE;
     }
     else {
         int read;
         int yes;
         int no;
         int i;
         int j;
         int sy;
         int p;
         int oldfont = FontGetSys();

         LCD_BUFFER LCD_tmp;
         LCD_BUFFER LCD_tmp2;

         if (nr_planes == 2) {
             HandleSetPlane(0);
             BitmapGet(ScrRect,LCD_tmp);
             HandleSetPlane(1);
             BitmapGet(ScrRect,LCD_tmp2);
         }
         else {
             LCD_save(LCD_tmp);
         }

         sy = ScrRect->xy.y1/2 - 2;
         for (i=0;i<=10;i++) {
             for (p=0;p<nr_planes;p++) {
                HandleSetPlane(p);
                DrawClipRect(MakeWinRect(0,sy-i,ScrRect->xy.x1-1,sy+i),ScrRect,1);
                DrawClipRect(MakeWinRect(1,sy-i+1,ScrRect->xy.x1-2,sy+i-1),ScrRect,0);
             }
             for (j=0;j<=4000;j++);
         }

         FontSetSys(F_6x8);

         for (p=0;p<nr_planes;p++) {
            HandleSetPlane(p);
            DrawStr((ScrRect->xy.x1-DrawStrWidth(request,F_6x8))/2,sy-3,request,A_REPLACE);
         }

         do {
             read = ngetchx();
             yes  = isYesKey(read);
             no   = isNoKey(read);
         }
         while (!yes && !no);

         if (nr_planes == 2) {
             HandleSetPlane(0);
             BitmapPut(0,0,LCD_tmp,ScrRect,A_REPLACE);
             HandleSetPlane(1);
             BitmapPut(0,0,LCD_tmp2,ScrRect,A_REPLACE);
         }
         else {
             LCD_restore(LCD_tmp);
         }

         FontSetSys(oldfont);
         return yes;
    }
}


/*===========================================================================*/
/* list all stored moves so far                                              */
/*===========================================================================*/
void ShowStoredMoves() {
    int  i  = 0;
    int  x  = 0;
    int  mv = move_count-moves_stored;
    char s1[10];
    char s2[10];
    char s3[10];
    char s4[10];

    if (moves_stored == 0) {
#if TI92_VERSION
       puts("No moves stored. Sorry.\n");
#else
       puts("No moves stored.\nSorry.\n");
#endif
       return;
    }

    puts("-- stored moves --\n");

    if (mv % 2 == 1) {
        printf("[%03d] ...... %s\n",mv,Move2Str((move_t*)&move_store[0],FALSE));
        i = 1;
    }

    for (;i<moves_stored;i+=2) {
#if TI92_VERSION
        printf("[%03d] %s  ",mv+i,Move2Str((move_t*)&move_store[i],FALSE));
#else
        printf("%03d %s ",mv+i,Move2Str((move_t*)&move_store[i],FALSE));
#endif
        if (i+1 < moves_stored) {
            puts(Move2Str((move_t*)&move_store[i+1],FALSE));
        }
        putc('\n');

        if ((i/2) % LINES_PER_PAGE == LINES_PER_PAGE-1) {
            if (i<(moves_stored-2)) {
                puts("--- PRESS KEY ---");
                x=ngetchx();
                putc('\n');
                if (x==KEY_ESCAPE) {
                    puts("listing aborted.\n");
                    return;
                }
            }
        }
    }
}


/*===========================================================================*/
/* handles a computer move                                                   */
/*===========================================================================*/
void HandleComputerMove() {
    move_t *executed = ComputerMove();
    UpdateNodeDisplay(nr_nodes);

    if (executed != 0) {
        marked_rect1 = executed->to;
        marked_rect2 = executed->from;
        if (!already_mate) {
            if (act_color == WHITE) {
                if (AttacksField(white_king_pos,BLACK)) puts("CHECK !!!\n");
            }
            else {
                if (AttacksField(black_king_pos,WHITE)) puts("CHECK !!!\n");
            }
        }
    }
    else {
        automatic_mode = FALSE;
    }
}



/*===========================================================================*/
/* nomen omen est                                                            */
/*===========================================================================*/
void MajorInputLoop() {
    int  read;
    int  tmpfig;
    int  i;
    char tmpnotation[10];

    do {
        DrawBoard(automatic_mode);
        if (choice == MENU_NONE) {
            if (!automatic_mode) {
                read = tolower(ngetchx());
                automatic_mode = FALSE;
            }
            else {
                // automatic mode goes here ...
                HandleComputerMove();
                continue;
            }
        }
        else {
            switch(choice) {
                case MENU_NEW:
                    longjmp(jmp_buf,1);
                    break;
                case MENU_PROBLEM:
                    if (HandleLoad(FALSE,TRUE)) {
                        printf("Puzzle %03d loaded.\n- %s\n- %s\n",problem_active,
                                problem_text2,
                                problem_text3);
                    }
                    else {
                        puts("Loading failed.\n");
                    }
                    choice = MENU_NONE;
                    continue;
                case MENU_CMD_LOAD:
                    if (HandleLoad(FALSE,FALSE)) puts("Successfully loaded.\n");
                    else                         puts("Loading failed.\n");
                    choice = MENU_NONE;
                    continue;
                case MENU_CMD_SAVE:
                    if (HandleSave(FALSE)) puts("Successfully saved.\n");
                    else                   puts("Saving failed.\n");
                    choice = MENU_NONE;
                    continue;
                case MENU_EXIT:
                    return;
                default:
                    choice = MENU_NONE;
                    continue;
            }
        }
        //printf("key=%d == [%c]\n",read,(char)read);

        switch(read) {
            case KEY_ESCAPE:
                choice = MainMenu(MENU_NONE);
                break;
            case KEY_OFF1:
            case KEY_OFF2:
                asm("trap #4");
                break;

            case KEY_LEFT:
                //if (select_x>0) {
                    // force redraw of previous
                    drawn[Cursor2Idx(select_x,select_y)] = -OUTSIDE;
                    select_x--;
                    if (select_x < 0) select_x = 7;
                //}
                break;

            case KEY_RIGHT:
                //if (select_x<7) {
                    // force redraw of previous
                    drawn[Cursor2Idx(select_x,select_y)] = -OUTSIDE;
                    select_x++;
                    if (select_x > 7) select_x = 0;
                //}
                break;

            case KEY_DOWN:
                //if (select_y<7) {
                    // force redraw of previous
                    drawn[Cursor2Idx(select_x,select_y)] = -OUTSIDE;
                    select_y++;
                    if (select_y > 7) select_y = 0;
                //}
                break;

            case KEY_UP:
                //if (select_y>0) {
                    // force redraw of previous
                    drawn[Cursor2Idx(select_x,select_y)] = -OUTSIDE;
                    select_y--;
                    if (select_y < 0) select_y = 7;
                //}
                break;

            case KEY_ENTER: // ENTER
#if TI89_VERSION
            case KEY_CLEAR:
            case KEY_HOME:
#else
            case KEY_F1:
#endif
                if (already_mate) break;
                if (idx_from == -1) {
                    i = Cursor2Idx(select_x,select_y);
                    tmpfig=board[i];
                    if (tmpfig == EMPTY ||
                       (act_color==WHITE && tmpfig<0) ||
                       (act_color==BLACK && tmpfig>0)) break;

                    printf("[%03d] ",move_count);
                    if (act_color == BLACK) puts("..........  ");
                    putc(Figure2Char(tmpfig));
                    puts(Field2Notation(i,tmpnotation));
                    idx_from = i;
                }
                else {
                    i = Cursor2Idx(select_x,select_y);
                    if (i == idx_from) {
                        putc('\r');
                        idx_from = -1;
                        break;
                    }
                    tmpfig=board[i];

                    if ((act_color==WHITE && tmpfig>0) ||
                        (act_color==BLACK && tmpfig<0)) break;

                    if (tmpfig==EMPTY) putc('-');
                    else               putc('x');
                    puts(Field2Notation(i,tmpnotation));
                    putc('\n');

                    idx_to = i;

                    tmpfig = board[idx_from];
                    if ((act_color==WHITE && tmpfig==W_PAWN && idx_from >= A7) ||
                        (act_color==BLACK && tmpfig==B_PAWN && idx_from <= H2))
                    {
                        puts("PROMOTE TO Q/R/N/B? ");
                        do {
                            select_promote = toupper(ngetchx());                            
#if TI89_VERSION
                            // prevent alpha keys on TI89
                            switch(select_promote) {
                                case '1': select_promote = 'Q'; break;
                                case '2': select_promote = 'R'; break;
                                case '6': select_promote = 'N'; break;
                                case '(': select_promote = 'B'; break;
                            }
#endif
                            
                        }
                        while (select_promote != KEY_ENTER  &&
                               select_promote != 'Q' &&
                               select_promote != 'R' &&
                               select_promote != 'N' &&
                               select_promote != 'B');

                        if (select_promote == KEY_ENTER) select_promote='Q';
                        putc(select_promote);
                        putc('\n');
                    }
                    else {
                       select_promote=0;
                    }


                    Field2Notation(idx_from,tmpnotation);
                    Field2Notation(idx_to,&tmpnotation[2]);
                    tmpnotation[4] = select_promote;
                    tmpnotation[5] = 0;

                    if (!UserMove(tmpnotation)) {
                        puts("ERROR: Invalid Move\n");
                    }
                    else {
                        if (!twoplayer_mode) {
                            DrawBoard(TRUE);
                            HandleComputerMove();
                        }
                    }
                    idx_from = -1;
                }
                break;

            case KEY_HELP:
                MainMenu(MENU_HELP);
                break;

            case KEY_AUTOMATIC:
                if (!automatic_mode) {
                   if (RequestDialog("Start Automatic (Y/N)?")) {
                      automatic_mode = TRUE;
                      twoplayer_mode = FALSE;
                   }
                }
                else {
                   automatic_mode = FALSE;
                   puts("Automatic stopped.\n");
                }
                break;

            case KEY_LOADSAVE:  // STO key
                choice = MainMenu(MENU_LOADSAVE);
                break;

            case 'b':
            case KEY_BACK: // backspace
                TakeBackMove(FALSE);
                already_mate = FALSE;
                break;

            case KEY_FORWARD:
                UndoTakeBackMove(FALSE);
                break;

            case 'c':
#if TI89_VERSION
            case ')':
#endif
                if (RequestDialog("Change Sides (Y/N)?")) {
                    puts("Sides changed.\n");
                    DrawBoard(TRUE);
                    HandleComputerMove();
                }
                break;

            case KEY_LEVELUP:
                std_depth++;
                if (std_depth > MAX_USERLEVEL) std_depth = MAX_USERLEVEL;
                printf("Now on Level %d.\n",std_depth-1);
                defaults.strength = std_depth-1;
                SaveDefaults();
                break;

            case KEY_LEVELDOWN:
                std_depth--;
                if (std_depth < MIN_USERLEVEL) std_depth = MIN_USERLEVEL;
                printf("Now on Level %d.\n",std_depth-1);
                defaults.strength = std_depth-1;
                SaveDefaults();
                break;

            case 'x':
                ShowStoredMoves();
                break;

#if DEVELOPER_VERSION
            case 'z':
                ToggleHashStatistics();
                break;

            case 'e':
                printf("eval=%d\n",Evaluate(-VALUE_MATE,VALUE_MATE,act_color));
                break;

            case 'm':
                ToggleHashMode();
                break;

            case 'o':
                printf("dpth=%d bal=%d sum=%d\n",act_depth,material_bal[0],material_sum[0]);
                break;

            case 'k':
                {
                hash_t hk = act_hashkey;
                puts(Hash2Str(act_hashkey));
                CalcHashKey();
                if (act_hashkey != hk) {
                   puts("!=");
                   puts(Hash2Str(act_hashkey));
                }
                }
                break;
#endif
            case '2':
                if (twoplayer_mode) {
                    if (RequestDialog("ONE Player (Y/N)?")) {
                        twoplayer_mode = FALSE;
                        puts("One Player Mode.\n");
                    }
                }
                else {
                    if (RequestDialog("TWO player (Y/N)?")) {
                        twoplayer_mode = TRUE;
                        puts("Two Player Mode.\n");
                    }
                }
                break;

#if TI89_VERSION
            case 't':
#else
            case 'r':
#endif
                if (inverted) inverted = FALSE;
                else          inverted = TRUE;

                for (i=0;i<BOARD_SIZE;i++) drawn[i] = -OUTSIDE;
                DrawCoordinates();
                DrawBoard(FALSE);
                break;
                
                
            //case 'e':
            //    while (moves_stored > 0) TakeBackMove(TRUE);
            //    break;
            
            //case 'w':
            //   while (moves_forward > 0) UndoTakeBackMove(TRUE);
            //   break;                

        }
    }
    while (1);
}



/*===========================================================================*/
/* main routine of the GUI                                                   */
/*===========================================================================*/
void MainGUI(void) {
    int p;
    do_exit = 0;
    choice  = MENU_NONE;

    if (defaults.autoload == OPTION_OFF || defaults.firsttime == OPTION_ON) {
        for (p=0;p<nr_planes;p++) {
            HandleSetPlane(p);
            ClrScr();
        }
        choice = MainMenu(MENU_NONE);
        defaults.firsttime = OPTION_OFF;
        SaveDefaults();
    }

    if (choice == MENU_EXIT) return;

    setjmp(jmp_buf);

    if (choice == MENU_CONTINUE) choice = MENU_NONE;

    if (do_exit == 0) {
        InitGUI();
        automatic_mode = FALSE;
        twoplayer_mode = FALSE;
        InitEngine(defaults.strength+1);

        if (choice != MENU_NEW             &&
            defaults.autoload == OPTION_ON &&
            HandleLoad(TRUE,FALSE))
        {
            printf("On Level %d.\nGame restored.\n",std_depth-1);
        }
        else {
            printf("On Level %d.\nTry to beat me!\n",std_depth-1);
        }

        if (choice == MENU_NEW) choice = MENU_NONE;
        MajorInputLoop(choice);
    }

    if (defaults.autosave == OPTION_ON) HandleSave(TRUE);
}


//##########################################################################
//###################### NO MORE FAKES BEYOND THIS LINE ####################
//##########################################################################
