/******************************************************************************
*
* project name:    TICHESS
* project version: 2.00
* file name:       loadsave.c
* initial date:    19/03/2000
* author:          thomas.nussbaumer@peem.com
* description:     contains load/save routines
*
* -----------------------------------------------------------------------------
* Revision History
* -----------------------------------------------------------------------------
*
* v1.07: 06/04/2000        
*        bug fixed which causes a crash if some moves are tacken back
*        and the program tries to save them
*
* v1.06: 03/04/2000        
*        saving now moves again
*
* v1.05: 28/03/2000        
*        handling for chess problem data base added
*
* v1.04: 24/03/2000        
*        material balance and sum will be calculated during save (not
*        stored in save file anymore)
*
* v1.03: 22/03/2000        
*        automatic uses now always slot ticsave0
*
* v1.02: 22/03/2000        
*        LoadDefaults() & SaveDefaults() moved from menu.c to here
*
* v1.01: 20/03/2000        
*        no ticauto anymore, but 4 save slots (and one is used for automatic)
*
* v1.00: 19/03/2000        
*        initial version ("ticauto" and "ticsave" are used)
*
******************************************************************************/


int problem_active = 0;
int problem_max    = 0;

char problem_text1[SIZE_TEXT+1] = {0};
char problem_text2[SIZE_TEXT+1] = {0};
char problem_text3[SIZE_TEXT+1] = {0};
char problem_copyright[SIZE_COPYRIGHT+1] = {0};


//-----------------------------------------------------------------------------
// declarations of the necessary "external" variables
//-----------------------------------------------------------------------------
extern int          updatecounter;
extern int          already_mate;
extern char*        board;
extern int*         ep_field;
extern int*         stack_ranges;
extern int*         material_bal;
extern int*         material_sum;
extern int          white_king_pos;
extern int          black_king_pos;
extern int          act_color;
extern int          act_depth;
extern long         nr_nodes;
extern unsigned int move_count;
extern int          moves_stored;
extern int          moves_forward;
extern unsigned int flags[MAX_DEPTH];
extern defaults_t   defaults;
extern store_t*     move_store;
extern const char   fig_symbols[NR_FIGS*2-1];
extern const int    fig_values[NR_FIGS];


extern menu_t ls_menu[2];

void   ResetHashCounters(void);
void   UpdateTurnDisplay(int color);
void   UpdateNodeDisplay(long nodes);
void   CalcHashKey();
void   InitPawnVars();
char*  Field2Notation(int fieldnumber,char* fieldnotation);
int    UserMove(char* move);
void   TakeBackMove(int silent);
void   UndoTakeBackMove(int silent);
//void   puts(const char* s);

/*===========================================================================*/
/* if automatic == TRUE: uses ticsave0 for loading                           */
/* else                  uses name from ls_menu (load/save menu)             */
/*===========================================================================*/
int HandleLoad(int automatic,int load_problem) {
    SYM_ENTRY*     symentry;
    unsigned char* varptr;
    int            i;
    int            idx;
    unsigned char  lo;
    unsigned char  hi;
    int            content;
    unsigned int   abs_val;
    int            moves = -1;

    unsigned char  use = 0;

    if (load_problem) {
        symentry = SymFindPtr ($(ticpro), 0);
    }
    else {
        if (!automatic) use = ls_menu[0].active;

        switch(use) {
            case 1:  symentry = SymFindPtr ($(ticsave1), 0); break;
            case 2:  symentry = SymFindPtr ($(ticsave2), 0); break;
            case 3:  symentry = SymFindPtr ($(ticsave3), 0); break;
            default: symentry = SymFindPtr ($(ticsave0), 0); break;
        }
    }
    
    if (!symentry) return FALSE;

    varptr = (unsigned char*)HeapDeref(symentry->handle);

    if (!varptr) return FALSE;


    if (!load_problem) {
        //char s[256];
        moves = ((int)varptr[0]) * 256 + ((int)varptr[1]) - 39;        
        if (moves % 3) return FALSE;
        moves /= 3;
        //sprintf(s,"moves=%d\n",moves);
        //puts(s);
    }
    else {
        varptr += SIZE_PROBLEM*(problem_active - 1) + SIZE_TEXT*3+SIZE_COPYRIGHT + 2;
    }    
        
    idx = A1;
    for (i=3;i<35;i++) {
        hi = (varptr[i] & 0xf0) >> 4;
        lo = (varptr[i] & 0x0f);

        if (hi == W_KING)         white_king_pos = idx;
        else if (hi == B_KING+13) black_king_pos = idx;

        if (lo == W_KING)         white_king_pos = idx+1;
        else if (lo == B_KING+13) black_king_pos = idx+1;

        if (hi > 6) board[idx] = ((char)hi) - 13;
        else        board[idx] = ((char)hi);
        idx++;
        if (lo > 6) board[idx] = ((char)lo) - 13;
        else        board[idx] = ((char)lo);
        idx++;
        if (idx%10==9) idx+=2;
    }

    move_count      = (unsigned int) ((varptr[35] << 8)| varptr[36]);
    flags[0]        = (unsigned int) ((varptr[37] << 8)| varptr[38]);

    if ((move_count % 2) == 0) act_color = WHITE;
    else                       act_color = BLACK;

    material_sum[0] = 0;
    material_bal[0] = 0;
        
    for (i=A1;i<=H8;i++) {
       content = board[i];
       
       if (content == OUTSIDE || content == EMPTY) continue;
       abs_val = abs(content);
       if (content < 0) {
           material_bal[0] -= fig_values[abs_val];
       }
       else {
           material_bal[0] += fig_values[abs_val];
       }

       if (abs_val == W_PAWN) continue;
       material_sum[0] += fig_values[abs_val];
    }
    
    if (act_color == BLACK) material_bal[0] = -material_bal[0];

    UpdateTurnDisplay(act_color);
    UpdateNodeDisplay(0);

    already_mate    = FALSE;
    updatecounter   = 0;
    moves_stored    = 0;
    act_depth       = 0;
    ep_field[0]     = ILLEGAL;
    stack_ranges[0] = 0;
    nr_nodes        = 0;

    CalcHashKey();
    InitPawnVars();
    ResetHashCounters();
           
    if (moves > 0) {
        char m_t[6];
        char from;
        char to;
        char promote;
        for (i=0;i<moves;i++) {
            from    = (char)varptr[39+i*3];
            to      = (char)varptr[39+i*3+1];
            promote = (char)varptr[39+i*3+2];
            
            Field2Notation((int)from,m_t);
            Field2Notation((int)to,&m_t[2]);
            if (promote != EMPTY) {
                m_t[4] = fig_symbols[promote+6];
                m_t[5] = 0;
            }
            else {
                m_t[4] = 0;
            }
            
            UserMove(m_t);
        }
    }
    
    return TRUE;
}

/*===========================================================================*/
/* if auto == TRUE: uses ticsave0 for saving                                 */
/* else             uses savename from ls_menu (load/save menu)              */
/*===========================================================================*/
int HandleSave(int automatic) {
    SYM_ENTRY*     symentry;
    unsigned char* varptr;
    int            i;
    int            idx;
    char           lo,hi;
    char           use   = 3;
    int            moves = 0;

    if (!automatic) use = ls_menu[1].active;

    switch(use) {
        case 0:  symentry = DerefSym( SymAdd( $(ticsave1) ) ); break;
        case 1:  symentry = DerefSym( SymAdd( $(ticsave2) ) ); break;
        case 2:  symentry = DerefSym( SymAdd( $(ticsave3) ) ); break;
        default: symentry = DerefSym( SymAdd( $(ticsave0) ) ); break;
    }

    if (!symentry) return FALSE;

    //varptr = (unsigned char*)HeapDeref(symentry->handle = HeapAlloc(40+5));

    if (defaults.savemoves == OPTION_ON) {
        moves = moves_stored;
        varptr = (unsigned char*)HeapDeref(symentry->handle = HeapAlloc(40+5+moves*3));
        idx = 36+3+moves*3;
    }
    else {
        varptr = (unsigned char*)HeapDeref(symentry->handle = HeapAlloc(40+5));    
        idx = 36+3;
    }

    if (!varptr) return FALSE;
    
    varptr[0] = (unsigned char)(idx/256);  // 2 bytes: length of the variable - 2
    varptr[1] = (unsigned char)(idx%256);
    varptr[2] = 0;    // zero marks a begining of the actual variable data

    if (defaults.savemoves == OPTION_ON) {
        // take back all stored moves and save initial position        
        while (moves_stored > 0) TakeBackMove(TRUE);
    }
    
    idx = 3;
    for (i=A1;i<H8;i+=2) {
        if (board[i] == OUTSIDE) continue;
        hi = board[i];
        lo = board[i+1];
        if (lo<0) lo += 13;
        if (hi<0) hi += 13;
        varptr[idx] = (hi & 0x0f) << 4;
        varptr[idx] |= (lo & 0x0f);
        idx++;
    }

    varptr[idx++] = (move_count & 0xff00) >> 8;
    varptr[idx++] = move_count & 0xff;
    varptr[idx++] = (flags[0] & 0xff00) >> 8;
    varptr[idx++] = flags[0] & 0xff;

    if (defaults.savemoves == OPTION_ON) {
        // forward taken moves again and save moves
        while (moves > 0) {
            UndoTakeBackMove(TRUE);
            moves--;
        }
    
        for (i=0;i<moves_stored;i++) {
            varptr[idx++] = (unsigned char)move_store[i].from;
            varptr[idx++] = (unsigned char)move_store[i].to;
            varptr[idx++] = (unsigned char)move_store[i].promoted_fig;
        }    
    }
     
    varptr[idx++] = 0;    // end-of-string marker
    varptr[idx]   = 0x2D; // the last byte is a type (0x2D = STR)

    return TRUE;
}


defaults_t default_defaults = { STD_USERLEVEL-1, // strength
                                OPTION_ON,       // autoload
                                OPTION_ON,       // autosave
                                OPTION_ON,       // hashtables
                                OPTION_ON,       // grayscale
                                OPTION_ON,       // firsttime
                                OPTION_ON,       // requests
                                OPTION_ON};      // save moves

/*===========================================================================*/
/* load defaults from String "ticcfg"                                        */
/*===========================================================================*/
int LoadDefaults(void) {
    SYM_ENTRY*     symentry = SymFindPtr ($(ticcfg), 0);
    unsigned char* varptr;

    if (!symentry) {
        memcpy(&defaults,&default_defaults,sizeof(defaults_t));
        SaveDefaults();
        return FALSE;
    }

    varptr = (unsigned char*)HeapDeref(symentry->handle);
    if (!varptr) {
        memcpy(&defaults,&default_defaults,sizeof(defaults_t));
        return FALSE;
    }

    if (varptr == 0     || 
        varptr[0] != 0  || 
        varptr[1] != sizeof(defaults_t)+3 || 
        varptr[2] != 0) 
    {
        memcpy(&defaults,&default_defaults,sizeof(defaults_t));
        return FALSE;
    }

    memcpy(&defaults,&varptr[3],sizeof(defaults_t));
    return TRUE;
}


/*===========================================================================*/
/* save defaults to String "ticcfg"                                          */
/*===========================================================================*/
int SaveDefaults(void) {
    SYM_ENTRY*     symentry = DerefSym( SymAdd( $(ticcfg) ) );
    unsigned char* varptr;

    if (!symentry) return FALSE;

    varptr = (unsigned char*)HeapDeref(symentry->handle = HeapAlloc(sizeof(defaults_t)+5));
    if (!varptr) return FALSE;

    varptr[0] = 0;    // 2 bytes: length of the variable - 2
    varptr[1] = sizeof(defaults_t)+3;
    varptr[2] = 0;    // zero marks a begining of the actual variable data

    memcpy(&varptr[3],&defaults,sizeof(defaults));

    varptr[3+sizeof(defaults_t)] = 0;    // end-of-string marker
    varptr[4+sizeof(defaults_t)] = 0x2D; // the last byte is a type (0x2D = STR)

    return TRUE;
}


/*===========================================================================*/
/* will load description of problem                                          */
/*===========================================================================*/
void LoadProblemDesc() {
    SYM_ENTRY*     symentry = SymFindPtr ($(ticpro), 0);
    unsigned char* varptr;
    unsigned int   tmp;

    if (!symentry) {
        problem_max = -1;
        return;
    }

    varptr = (unsigned char*)HeapDeref(symentry->handle);
    if (!varptr) {
        problem_max = -1;
        return;
    }

    varptr+=3;
    
    tmp = *varptr << 8;
    varptr++;
    tmp += *varptr;
    problem_max = (int)tmp;
    varptr++;

    if (problem_max == 0) {
        problem_max = -1;
        return;
    }

    if (problem_active > problem_max) problem_active = problem_max;
    else if (problem_active <= 0) problem_active = 1;
    
    varptr += SIZE_PROBLEM*(problem_active-1);

    memset(problem_text1,0,SIZE_TEXT+1);
    memset(problem_text2,0,SIZE_TEXT+1);
    memset(problem_text3,0,SIZE_TEXT+1);
    memset(problem_copyright,0,SIZE_COPYRIGHT+1);

    strncpy(problem_text1,varptr,SIZE_TEXT);
    varptr += SIZE_TEXT;
    strncpy(problem_text2,varptr,SIZE_TEXT);
    varptr += SIZE_TEXT;
    strncpy(problem_text3,varptr,SIZE_TEXT);
    varptr += SIZE_TEXT;
    strncpy(problem_copyright,varptr,SIZE_COPYRIGHT);
}


//##########################################################################
//###################### NO MORE FAKES BEYOND THIS LINE ####################
//##########################################################################
