/******************************************************************************
*
* project name:    TICHESS
* project version: 2.00
* file name:       menu.c
* initial date:    19/03/2000
* author:          thomas.nussbaumer@peem.com
* description:     contains the menu system
*
* -----------------------------------------------------------------------------
* Revision History
* -----------------------------------------------------------------------------
* v1.06: 03/04/2000
*        "pressing up in menu screen won't reach exit" bug fixed
*        Uppercase 'Y' in train your brain
*        "save moves" added to settings
*
* v1.05: 28/03/2000
*        support for chess problems added
*
* v1.04: 27/03/2000
*        load and save combined to a load/save dialog
*        [ESC] now won't quit program, but goes back to game window
*        auto name removed from settings (ticsave 0 will used always)
*
* v1.03: 24/03/2000
*        now using popups for menus
*
* v1.02: 22/03/2000
*        new settings entry "Requests" added to disable the Y/N questions
*        credits text finished
*
* v1.01: 20/03/2000
*        help menu and credits menu added
*        load and save menu added
*        autoname added to defaults
*
* v1.00: 19/03/2000
*        initial version
*
******************************************************************************/

#if TI92_VERSION

//-----------------------------------------------------------------------------
// constants for TI92 version
//-----------------------------------------------------------------------------
#define M_TSTART_Y     19
#define M_CSTART_Y     17
#define M_DELTA_Y      10
#define M_CENTERX     120
#define M_CENTERY      57
#define M_HEIGHT       84
#define M_HEIGHT2      42      
#define M_WIDTH       140
#define M_WIDTH2       70
#define M_PARAMS1X     55
#define M_PARAMS2X    120
#define M_CSTART_X     53
#define M_CEND_X      187
#define HP_START1X     55
#define HP_START2X    110
#define HP_DELTAY       7
#define CREDIT_ADDY     2

#else

//-----------------------------------------------------------------------------
// constants for TI89 version
//-----------------------------------------------------------------------------

#define M_TSTART_Y      9
#define M_CSTART_Y      7
#define M_DELTA_Y      10
#define M_CENTERX      80      
#define M_CENTERY      47
#define M_HEIGHT       84
#define M_HEIGHT2      42      
#define M_WIDTH       140
#define M_WIDTH2       70
#define M_CSTART_X     13
#define M_CEND_X      147
#define M_PARAMS1X     15
#define M_PARAMS2X     80
#define HP_START1X     15
#define HP_START2X     70
#define HP_DELTAY       7
#define CREDIT_ADDY     2

#endif


defaults_t defaults   = {};
int        option_idx = 0;
int        ls_idx     = 0;
int        show_idx   = 0;

extern int  std_depth;          // standard depth
extern int  LoadDefaults(void);
extern int  SaveDefaults(void);
extern int  isYesKey(int key);
extern int  isNoKey(int key);

int         ShowMenu(menu_t* pmenu,int nr);
void        HelpMenu(void);
void        CreditsMenu(void);
void        OptionMenu(void);
int         ProblemMenu(void);


extern int problem_active;
extern int problem_max;

extern char problem_text1[SIZE_TEXT+1];
extern char problem_text2[SIZE_TEXT+1];
extern char problem_text3[SIZE_TEXT+1];
extern char problem_copyright[SIZE_COPYRIGHT+1];

extern void LoadProblemDesc();


/*===========================================================================*/
/* draws center string                                                       */
/*===========================================================================*/
void DrawCenterString(int y,char* s,int font,int attr) {
    int p;
    
    for (p=0;p<nr_planes;p++) {
        HandleSetPlane(p);
        DrawStr((ScrRect->xy.x1-DrawStrWidth(s,font))/2,y,s,attr);
    }
}

/*===========================================================================*/
/* draws center string                                                       */
/*===========================================================================*/
void DrawStr2(int x,int y,char* s,int attr) {
    int p;
    
    for (p=0;p<nr_planes;p++) {
        HandleSetPlane(p);
        DrawStr(x,y,s,attr);
    }
}

/*===========================================================================*/
/* draws cursor by inverting an entry                                        */
/*===========================================================================*/
void DrawCursor(int idx) {
    int p;
    SCR_RECT sr = {M_CSTART_X,M_DELTA_Y*idx+M_CSTART_Y,
                   M_CEND_X,M_DELTA_Y*(idx+1)+M_CSTART_Y};

    for (p=0;p<nr_planes;p++) {
        HandleSetPlane(p);
        ScrRectFill(&sr,ScrRect,A_XOR);
    }
}

/*===========================================================================*/
/* clears menu area                                                          */
/*===========================================================================*/
void ClearMenu() {
    int p;
    SCR_RECT sr = {M_CENTERX-M_WIDTH2+2,M_CENTERY-M_HEIGHT2+2,
                   M_CENTERX+M_WIDTH2-2,M_CENTERY+M_HEIGHT2-2};
                   
    for (p=0;p<nr_planes;p++) {
        HandleSetPlane(p);
        ScrRectFill(&sr,ScrRect,A_REVERSE);
    }
}

#define OPT1ARRAY {"level 0","level 1","level 2","level 3","level 4"}
#define OPT2ARRAY {"on","off"}
#define OPT3ARRAY {"grayscale","monochrome"}
#define OPT4ARRAY {"ticsave0","ticsave1","ticsave2","ticsave3"}
#define OPT5ARRAY {"ticsave1","ticsave2","ticsave3"}

menu_t settings[7] = {
{"STRENGTH",   5, OPT1ARRAY, 0, "",FALSE},
{"AUTOLOAD",   2, OPT2ARRAY, 0, "",FALSE},
{"AUTOSAVE",   2, OPT2ARRAY, 0, "",FALSE},
{"HASHING",    2, OPT2ARRAY, 0, "[restart if changed]",FALSE},
{"GRAPHICS",   2, OPT3ARRAY, 0, "[restart if changed]",FALSE},
{"REQUESTS",   2, OPT2ARRAY, 0, "[toggles Y/N requests]",FALSE},
{"SAVE MVS",   2, OPT2ARRAY, 0, "",FALSE}
};

menu_t ls_menu[2] = {
{"LOAD GAME",  4, OPT4ARRAY, 0, "[press ENTER to LOAD]",TRUE},
{"SAVE GAME",  3, OPT5ARRAY, 0, "[press ENTER to SAVE]",TRUE}
};


char* main_entries[] = { "NEW Game","LOAD/SAVE Game","TRAIN Your Brain","SETTINGS","CONTINUE","HELP","CREDITS","EXIT" };
int   main_idx = 0;

/*===========================================================================*/
/* show a menu                                                               */
/*===========================================================================*/
int MainMenu(int showmenu) {
    int        i;
    int        read;
    int        end_of_loop = FALSE;
    int        redraw_menu = TRUE;
    int        oldfont     = FontGetSys();
    LCD_BUFFER lcdbuf;
    LCD_BUFFER lcdbuf2;
    int        p;
    int        y;
    int        retval;

    if (nr_planes == 2) {
        HandleSetPlane(0);
        BitmapGet(ScrRect,lcdbuf);
        HandleSetPlane(1);
        BitmapGet(ScrRect,lcdbuf2);
    }
    else {
        LCD_save(lcdbuf);
    }


    for (i=0;i<M_WIDTH2;i++) {
        for (p=0;p<nr_planes;p++) {
            HandleSetPlane(p);
            if (i < M_HEIGHT2) y = i;
            DrawClipRect(MakeWinRect(M_CENTERX-i,M_CENTERY-y,M_CENTERX+i,M_CENTERY+y),ScrRect,1);
            DrawClipRect(MakeWinRect(M_CENTERX-i+1,M_CENTERY-y+1,M_CENTERX+i-1,M_CENTERY+y-1),ScrRect,0);
        }
    }


    if (showmenu != MENU_NONE) {
        FontSetSys(F_6x8);
        switch(showmenu) {
            case MENU_LOADSAVE:
                show_idx = ls_idx;
                if (ShowMenu(ls_menu,2)) {
                    if (show_idx == 0) {
                       retval = MENU_CMD_LOAD;
                    }
                    else {
                       retval = MENU_CMD_SAVE;
                    }
                }
                else {
                    retval = MENU_CONTINUE;
                }
                ls_idx = show_idx;
                break;
            case MENU_HELP:
                HelpMenu();
                retval = MENU_CONTINUE;
                break;
        }
    }
    else {
        do {
            if (redraw_menu) {
                FontSetSys(F_6x8);
                ClearMenu();

                for (i=0;i<8;i++) DrawCenterString(M_DELTA_Y*i+M_TSTART_Y,main_entries[i],F_6x8,A_REPLACE);

                DrawCursor(main_idx);
                redraw_menu = FALSE;
            }
            read = tolower(ngetchx());
            switch(read) {
                case KEY_UP:
                    DrawCursor(main_idx);
                    main_idx--;
                    if (main_idx<0) main_idx=7;
                    DrawCursor(main_idx);
                    break;
                case KEY_DOWN:
                    DrawCursor(main_idx);
                    main_idx++;
                    if (main_idx>7) main_idx=0;
                    DrawCursor(main_idx);
                    break;
                case 'x':
                    retval = MENU_EXIT;
                    end_of_loop = TRUE;
                    break;
                case KEY_ENTER: // ENTER
#if TI89_VERSION
                case KEY_CLEAR:
                case KEY_HOME:
#else 
                case 268:
#endif
                    switch(main_idx) {
                        case MENU_SETTINGS: 
                            OptionMenu();
                            redraw_menu = TRUE;
                            break;
                        case MENU_LOADSAVE: 
                            show_idx = ls_idx;
                            if (ShowMenu(ls_menu,2)) {
                                if (show_idx == 0) {
                                    retval = MENU_CMD_LOAD;
                                }
                                else {
                                    retval = MENU_CMD_SAVE;
                                }
                                end_of_loop = TRUE;
                            }
                            else {
                                redraw_menu = TRUE;
                            }
                            ls_idx = show_idx;
                            break;
                        case MENU_PROBLEM:
                            if (ProblemMenu()) {
                                retval = MENU_PROBLEM;
                                end_of_loop = TRUE;
                            }
                            else {
                                redraw_menu = TRUE;
                            }
                            break;

                        case MENU_CREDITS:
                            CreditsMenu();
                            redraw_menu = TRUE;
                            break;
                        case MENU_HELP:
                            HelpMenu();
                            redraw_menu = TRUE;
                            break;
                        default: 
                            retval = main_idx;
                            end_of_loop = TRUE;
                            break;
                    }
                    break;
            }
        
        } 
        while(read != KEY_ESCAPE && !end_of_loop);
    }

    if (nr_planes == 2) {
        HandleSetPlane(0);
        BitmapPut(0,0,lcdbuf,ScrRect,A_REPLACE);
        HandleSetPlane(1);
        BitmapPut(0,0,lcdbuf2,ScrRect,A_REPLACE);
    }
    else {
        LCD_restore(lcdbuf);
    }

    FontSetSys(oldfont);

    if (read == KEY_ESCAPE)           return MENU_NONE;
    else if (retval == MENU_CONTINUE) return MENU_NONE;

    return retval;
}


#if TI92_VERSION

//--------------------------------------------
// help page for TI92 version
//--------------------------------------------
const char* helppage[] =
        { "[STO]",       "Load/Save Game",
          "[C]",         "Change Sides",
          "[+][-]",      "Level Up/Down",
          "[A]",         "Toggle Automatic Play",
          "[X]",         "Show Moves so far",
          "[BACK][B]",   "Take Back a Move",
          "[F]",         "Undo Take Back",
          "[R]",         "Rotate board by 180",
          "[2]",         "Toggle 2-Player Mode",
          "[F1]",        "==[ENTER]==[Y]",
          "[ESC]",       "==[N] or Main Menu",
          0};

#else

//--------------------------------------------
// help page for TI89 version
//--------------------------------------------
const char* helppage[] =
        { 
          "[STO]",     "Load/Save Game",
          "[C][)]",    "Change Sides",
          "[+][-]",    "Level Up/Down",
          "[0]",       "Automatic Play",
          "[X]",       "Show Moves so Far",
          "[BACK]",    "Take Back a Move",
          "[CATALOG]", "Undo Take Back",
          "[T]",       "Rotate Board",
          "[2]",       "2-Player Mode",
          "[HOME]",    "=[ENTER]=[CLR]=[Y]",
          "[ESC]",     "=[N] or Main Menu",
          0};
#endif


const char* creditspage[] =
        { 
          "------ AND THE CREDITS GO TO ------",
          "Thomas Nussbaumer (coding)",
          "Adrian Bader & Simon Brady",
          "K.+ M.Csillag & J. Clifford",
          "Nik Doytchinov & Ralf Fichtner",
          "Zeljko Juric & Marcos Lopez",
          "James Marshall & David Mejest",
          "Manu Raam & Birgit Tatzel",
          "Xavier Vassor & Peter Vrabel",
          "------- SPECIAL THANX TO -------",
          "everyone who sent suggestions",
          0};
         
/*
const char* creditspage[] =
        { 
          "---- COORDINATION & CODING ----",
          "Thomas Nussbaumer",
          "-------- PIECE GRAPHICS --------",
          "Marianna Csillag & Marcos Lopez",
          "-------- BETA TESTING ---------",
          "Ralf Fichtner,Brady Simon,Manu Raam",
          "------- TIGCC and C STDLIB -------",
          "Xavier Vassor & Zeljko Juric",
          "------- SPECIAL THANX TO -------",
          "everyone who sent suggestions",
          0};
*/

/*===========================================================================*/
/* handles help menu                                                         */
/*===========================================================================*/
void HelpMenu(void) {
    int i=0;

    ClearMenu();
    FontSetSys(F_4x6);

    do {
        DrawStr2(HP_START1X,i/2*HP_DELTAY+M_TSTART_Y,(char*)helppage[i],A_REPLACE);
        DrawStr2(HP_START2X,i/2*HP_DELTAY+M_TSTART_Y,(char*)helppage[i+1],A_REPLACE);
        i+=2;
    } 
    while (helppage[i] != 0);
    ngetchx();
}


/*===========================================================================*/
/* handles credits menu                                                      */
/*===========================================================================*/
void CreditsMenu(void) {
    int i=0;
    int y;
    int p;

    ClearMenu();
    FontSetSys(F_4x6);

    y=0;
    do {
        DrawStr2(M_CENTERX-DrawStrWidth((char*)creditspage[i],F_4x6)/2,
                 y+M_TSTART_Y,(char*)creditspage[i],A_REPLACE);
        y+=HP_DELTAY;
        //if (i%2 == 1) y += CREDIT_ADDY;
        i++;
    } 
    while (creditspage[i] != 0);
    ngetchx();
}


/*===========================================================================*/
/* handles problem menu                                                      */
/*===========================================================================*/
int ProblemMenu(void) {
    int  read;
    int  redraw = TRUE;
    char tmpbuf[100];
        
    do {
        if (redraw) {
            ClearMenu();
            LoadProblemDesc();

            if (problem_max == -1) {        
                DrawStr2(M_PARAMS1X,M_TSTART_Y,"Problems File Missing",A_REPLACE);
                DrawStr2(M_PARAMS1X,M_DELTA_Y*2+M_TSTART_Y,"Please Upload TICPRO",A_REPLACE);
                ngetchx();
                return FALSE;
            }

            DrawStr2(M_PARAMS1X,M_TSTART_Y,"Load Puzzle Nr:",A_REPLACE);
            sprintf(tmpbuf,"%03d",problem_active);
            DrawStr2(M_PARAMS2X+30,M_TSTART_Y,tmpbuf,A_REPLACE);
            DrawCursor(0);
            DrawCenterString(M_DELTA_Y*2+M_TSTART_Y,problem_text1,F_6x8,A_REPLACE);
            DrawCenterString(M_DELTA_Y*3+M_TSTART_Y,problem_text2,F_6x8,A_REPLACE);
            DrawCenterString(M_DELTA_Y*4+M_TSTART_Y,problem_text3,F_6x8,A_REPLACE);
            FontSetSys(F_4x6);
            DrawCenterString(M_DELTA_Y*7+M_TSTART_Y,problem_copyright,F_4x6,A_REPLACE);
            FontSetSys(F_6x8);
            redraw = FALSE;
        }
        read = ngetchx();
           
        switch(read) {
            case KEY_RIGHT:
                problem_active++;
                if (problem_active > problem_max) {
                    problem_active = 1;
                }
                redraw = TRUE;
                break;
            case KEY_LEFT: 
                problem_active--;
                if (problem_active < 1) {
                    problem_active = problem_max;
                }
                redraw = TRUE;
                break;            
        }        
    } 
    while (!isYesKey(read) && !isNoKey(read));

    return isYesKey(read);
}


/*===========================================================================*/
/* handles option menu                                                       */
/*===========================================================================*/
void OptionMenu(void) {
    settings[0].active = defaults.strength;
    settings[1].active = defaults.autoload;
    settings[2].active = defaults.autosave;
    settings[3].active = defaults.hashtables;
    settings[4].active = defaults.graphics;
    settings[5].active = defaults.requests;
    settings[6].active = defaults.savemoves;
    show_idx = option_idx;
    ShowMenu(settings,7);
    option_idx = show_idx;

    defaults.strength   = settings[0].active;
    defaults.autoload   = settings[1].active;
    defaults.autosave   = settings[2].active;
    defaults.hashtables = settings[3].active;
    defaults.graphics   = settings[4].active;
    defaults.requests   = settings[5].active;
    defaults.savemoves  = settings[6].active;
  
    SaveDefaults(); // save defaults to "ticcfg"

    std_depth = defaults.strength+1;
}


/*===========================================================================*/
/* show a menu                                                               */
/*===========================================================================*/
int ShowMenu(menu_t* pmenu,int nr) {
    int i;
    int read;
    int p;
    
    ClearMenu();

    for (i=0;i<nr;i++) {
        DrawStr2(M_PARAMS1X,M_DELTA_Y*i+M_TSTART_Y,pmenu[i].name,A_REPLACE);
        if (pmenu[i].nr_options > 0) {
            DrawStr2(M_PARAMS2X,M_DELTA_Y*i+M_TSTART_Y,pmenu[i].options[pmenu[i].active],A_REPLACE);
        }
    }

    DrawCursor(show_idx);
    DrawStr2(M_PARAMS1X,M_DELTA_Y*7+M_TSTART_Y,pmenu[show_idx].help,A_REPLACE);

    do {
        read = ngetchx();

        switch(read) {
            case KEY_UP:
               DrawCursor(show_idx);
               DrawStr2(M_PARAMS1X,M_DELTA_Y*7+M_TSTART_Y,pmenu[show_idx].help,A_XOR);
               show_idx--;
               if (show_idx<0) show_idx=nr-1;
               DrawCursor(show_idx); 
               DrawStr2(M_PARAMS1X,M_DELTA_Y*7+M_TSTART_Y,pmenu[show_idx].help,A_REPLACE);
               break;
            case KEY_DOWN:
               DrawCursor(show_idx);
               DrawStr2(M_PARAMS1X,M_DELTA_Y*7+M_TSTART_Y,pmenu[show_idx].help,A_XOR);
               show_idx++;
               if (show_idx>=nr) show_idx=0;
               DrawCursor(show_idx);
               DrawStr2(M_PARAMS1X,M_DELTA_Y*7+M_TSTART_Y,pmenu[show_idx].help,A_REPLACE);
               break;

            case KEY_RIGHT:
               if (pmenu[show_idx].nr_options > 0) {
                   DrawStr2(M_PARAMS2X,M_DELTA_Y*show_idx+M_TSTART_Y,pmenu[show_idx].options[pmenu[show_idx].active],A_XOR);
                   pmenu[show_idx].active++;
                   if (pmenu[show_idx].active >= pmenu[show_idx].nr_options) pmenu[show_idx].active = 0;
                   DrawStr2(M_PARAMS2X,M_DELTA_Y*show_idx+M_TSTART_Y,pmenu[show_idx].options[pmenu[show_idx].active],A_REVERSE);
               }
               break;

            case KEY_LEFT:
               if (pmenu[show_idx].nr_options > 0) {
                   DrawStr2(M_PARAMS2X,M_DELTA_Y*show_idx+M_TSTART_Y,pmenu[show_idx].options[pmenu[show_idx].active],A_XOR);
                   pmenu[show_idx].active--;
                   if (pmenu[show_idx].active < 0) pmenu[show_idx].active = pmenu[show_idx].nr_options-1;
                   DrawStr2(M_PARAMS2X,M_DELTA_Y*show_idx+M_TSTART_Y,pmenu[show_idx].options[pmenu[show_idx].active],A_REVERSE);
               }
               break;
            case KEY_ENTER: // ENTER
#if TI89_VERSION
            case KEY_CLEAR:
            case KEY_HOME:
#else 
            case 268:
#endif
               if (pmenu[show_idx].isbutton) return TRUE;
               break;
        }
    } 
    while(read != KEY_ESCAPE);
    return FALSE;
}


//##########################################################################
//###################### NO MORE FAKES BEYOND THIS LINE ####################
//##########################################################################
